@@ -20,6 +20,7 @@ import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.headers.HeaderReader;
 import net.lingala.zip4j.headers.HeaderUtil;
 import net.lingala.zip4j.headers.HeaderWriter;
+import net.lingala.zip4j.io.inputstream.NumberedSplitRandomAccessFile;
 import net.lingala.zip4j.io.inputstream.ZipInputStream;
 import net.lingala.zip4j.model.FileHeader;
 import net.lingala.zip4j.model.ZipModel;
@@ -32,17 +33,21 @@ import net.lingala.zip4j.tasks.AddFolderToZipTask;
 import net.lingala.zip4j.tasks.AddFolderToZipTask.AddFolderToZipTaskParameters;
 import net.lingala.zip4j.tasks.AddStreamToZipTask;
 import net.lingala.zip4j.tasks.AddStreamToZipTask.AddStreamToZipTaskParameters;
+import net.lingala.zip4j.tasks.AsyncZipTask;
 import net.lingala.zip4j.tasks.ExtractAllFilesTask;
 import net.lingala.zip4j.tasks.ExtractAllFilesTask.ExtractAllFilesTaskParameters;
 import net.lingala.zip4j.tasks.ExtractFileTask;
 import net.lingala.zip4j.tasks.ExtractFileTask.ExtractFileTaskParameters;
 import net.lingala.zip4j.tasks.MergeSplitZipFileTask;
 import net.lingala.zip4j.tasks.MergeSplitZipFileTask.MergeSplitZipFileTaskParameters;
-import net.lingala.zip4j.tasks.RemoveEntryFromZipFileTask;
-import net.lingala.zip4j.tasks.RemoveEntryFromZipFileTask.RemoveEntryFromZipFileTaskParameters;
+import net.lingala.zip4j.tasks.RemoveFilesFromZipTask;
+import net.lingala.zip4j.tasks.RemoveFilesFromZipTask.RemoveFilesFromZipTaskParameters;
+import net.lingala.zip4j.tasks.RenameFilesTask;
+import net.lingala.zip4j.tasks.RenameFilesTask.RenameFilesTaskParameters;
 import net.lingala.zip4j.tasks.SetCommentTask;
 import net.lingala.zip4j.tasks.SetCommentTask.SetCommentTaskTaskParameters;
 import net.lingala.zip4j.util.FileUtils;
+import net.lingala.zip4j.util.RawIO;
 import net.lingala.zip4j.util.Zip4jUtil;
 
 import java.io.File;
@@ -52,8 +57,13 @@ import java.io.RandomAccessFile;
 import java.nio.charset.Charset;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.ThreadFactory;
 
 import static net.lingala.zip4j.util.FileUtils.assertFilesExist;
+import static net.lingala.zip4j.util.FileUtils.isNumberedSplitFile;
 import static net.lingala.zip4j.util.InternalZipConstants.CHARSET_UTF_8;
 import static net.lingala.zip4j.util.UnzipUtil.createZipInputStream;
 import static net.lingala.zip4j.util.Zip4jUtil.isStringNotNullAndNotEmpty;
@@ -80,6 +90,8 @@ public class ZipFile {
   private char[] password;
   private HeaderWriter headerWriter = new HeaderWriter();
   private Charset charset = CHARSET_UTF_8;
+  private ThreadFactory threadFactory;
+  private ExecutorService executorService;
 
   /**
    * Creates a new ZipFile instance with the zip file at the location specified in zipFile.
@@ -149,7 +161,7 @@ public class ZipFile {
     zipModel.setSplitArchive(splitArchive);
     zipModel.setSplitLength(splitLength);
 
-    new AddFilesToZipTask(progressMonitor, runInThread, zipModel, password, headerWriter).execute(
+    new AddFilesToZipTask(zipModel, password, headerWriter, buildAsyncParameters()).execute(
         new AddFilesToZipTaskParameters(filesToAdd, parameters, charset));
   }
 
@@ -169,7 +181,6 @@ public class ZipFile {
    */
   public void createSplitZipFileFromFolder(File folderToAdd, ZipParameters parameters, boolean splitArchive,
                                       long splitLength) throws ZipException {
-
     if (folderToAdd == null) {
       throw new ZipException("folderToAdd is null, cannot create zip file from folder");
     }
@@ -290,7 +301,7 @@ public class ZipFile {
       throw new ZipException("Zip file already exists. Zip file format does not allow updating split/spanned files");
     }
 
-    new AddFilesToZipTask(progressMonitor, runInThread, zipModel, password, headerWriter).execute(
+    new AddFilesToZipTask(zipModel, password, headerWriter, buildAsyncParameters()).execute(
         new AddFilesToZipTaskParameters(filesToAdd, parameters, charset));
   }
 
@@ -362,7 +373,7 @@ public class ZipFile {
       }
     }
 
-    new AddFolderToZipTask(progressMonitor, runInThread, zipModel, password, headerWriter).execute(
+    new AddFolderToZipTask(zipModel, password, headerWriter, buildAsyncParameters()).execute(
         new AddFolderToZipTaskParameters(folderToAdd, zipParameters, charset));
   }
 
@@ -399,7 +410,7 @@ public class ZipFile {
       throw new ZipException("Zip file already exists. Zip file format does not allow updating split/spanned files");
     }
 
-    new AddStreamToZipTask(progressMonitor, runInThread, zipModel, password, headerWriter).execute(
+    new AddStreamToZipTask(zipModel, password, headerWriter, buildAsyncParameters()).execute(
         new AddStreamToZipTaskParameters(inputStream, parameters, charset));
   }
 
@@ -434,7 +445,7 @@ public class ZipFile {
       throw new ZipException("invalid operation - Zip4j is in busy state");
     }
 
-    new ExtractAllFilesTask(progressMonitor, runInThread, zipModel, password).execute(
+    new ExtractAllFilesTask(zipModel, password, buildAsyncParameters()).execute(
         new ExtractAllFilesTaskParameters(destinationPath, charset));
   }
 
@@ -442,9 +453,7 @@ public class ZipFile {
    * Extracts a specific file from the zip file to the destination path.
    * If destination path is invalid, then this method throws an exception.
    * <br><br>
-   * If newFileName is not null or empty, newly created file name will be replaced by
-   * the value in newFileName. If this value is null, then the file name will be the
-   * value in FileHeader.getFileName
+   * If fileHeader is a directory, this method extracts all files under this directory
    *
    * @param fileHeader
    * @param destinationPath
@@ -457,6 +466,13 @@ public class ZipFile {
   /**
    * Extracts a specific file from the zip file to the destination path.
    * If destination path is invalid, then this method throws an exception.
+   * <br><br>
+   * If newFileName is not null or empty, newly created file name will be replaced by
+   * the value in newFileName. If this value is null, then the file name will be the
+   * value in FileHeader.getFileName. If file being extract is a directory, the directory name
+   * will be replaced with the newFileName
+   * <br><br>
+   * If fileHeader is a directory, this method extracts all files under this directory.
    *
    * @param fileHeader
    * @param destinationPath
@@ -478,7 +494,7 @@ public class ZipFile {
 
     readZipInfo();
 
-    new ExtractFileTask(progressMonitor, runInThread, zipModel, password).execute(
+    new ExtractFileTask(zipModel, password, buildAsyncParameters()).execute(
         new ExtractFileTaskParameters(destinationPath, fileHeader, newFileName, charset));
   }
 
@@ -491,8 +507,10 @@ public class ZipFile {
    * example is if there is a file "b.txt" in a folder "abc" in the zip file, then the
    * input file name has to be abc/b.txt
    * <br><br>
-   * Throws an exception if file header could not be found for the given file name or if
-   * the destination path is invalid
+   * If fileHeader is a directory, this method extracts all files under this directory.
+   * <br><br>
+   * Throws an exception of type {@link ZipException.Type#FILE_NOT_FOUND} if file header could not be found for the given file name.
+   * Throws an exception if the destination path is invalid.
    *
    * @param fileName
    * @param destinationPath
@@ -513,10 +531,13 @@ public class ZipFile {
    * <br><br>
    * If newFileName is not null or empty, newly created file name will be replaced by
    * the value in newFileName. If this value is null, then the file name will be the
-   * value in FileHeader.getFileName
+   * value in FileHeader.getFileName. If file being extract is a directory, the directory name
+   * will be replaced with the newFileName
    * <br><br>
-   * Throws an exception if file header could not be found for the given file name or if
-   * the destination path is invalid
+   * If fileHeader is a directory, this method extracts all files under this directory.
+   * <br><br>
+   * Throws an exception of type {@link ZipException.Type#FILE_NOT_FOUND} if file header could not be found for the given file name.
+   * Throws an exception if the destination path is invalid.
    *
    * @param fileName
    * @param destinationPath
@@ -534,7 +555,7 @@ public class ZipFile {
     FileHeader fileHeader = HeaderUtil.getFileHeader(zipModel, fileName);
 
     if (fileHeader == null) {
-      throw new ZipException("No file found with name " + fileName + " in zip file");
+      throw new ZipException("No file found with name " + fileName + " in zip file", ZipException.Type.FILE_NOT_FOUND);
     }
 
     extractFile(fileHeader, destinationPath, newFileName);
@@ -624,41 +645,13 @@ public class ZipFile {
   }
 
   /**
-   * Removes the file provided in the input parameters from the zip file.
-   * This method first finds the file header and then removes the file.
-   * If file does not exist, then this method throws an exception.
+   * Removes the file provided in the input file header from the zip file.
+   *
    * If zip file is a split zip file, then this method throws an exception as
    * zip specification does not allow for updating split zip archives.
    *
-   * @param fileName
-   * @throws ZipException
-   */
-  public void removeFile(String fileName) throws ZipException {
-
-    if (!isStringNotNullAndNotEmpty(fileName)) {
-      throw new ZipException("file name is empty or null, cannot remove file");
-    }
-
-    if (zipModel == null) {
-      readZipInfo();
-    }
-
-    if (zipModel.isSplitArchive()) {
-      throw new ZipException("Zip file format does not allow updating split/spanned files");
-    }
-
-    FileHeader fileHeader = HeaderUtil.getFileHeader(zipModel, fileName);
-    if (fileHeader == null) {
-      throw new ZipException("could not find file header for file: " + fileName);
-    }
-
-    removeFile(fileHeader);
-  }
-
-  /**
-   * Removes the file provided in the input file header from the zip file.
-   * If zip file is a split zip file, then this method throws an exception as
-   * zip specification does not allow for updating split zip archives.
+   * If this file header is a directory, all files and directories
+   * under this directory will be removed as well.
    *
    * @param fileHeader
    * @throws ZipException
@@ -668,6 +661,53 @@ public class ZipFile {
       throw new ZipException("input file header is null, cannot remove file");
     }
 
+    removeFile(fileHeader.getFileName());
+  }
+
+  /**
+   * Removes the file provided in the input parameters from the zip file.
+   * This method first finds the file header and then removes the file.
+   *
+   * If file does not exist, then this method throws an exception.
+   *
+   * If zip file is a split zip file, then this method throws an exception as
+   * zip specification does not allow for updating split zip archives.
+   *
+   * If the entry representing this file name is a directory, all files and directories
+   * under this directory will be removed as well.
+   *
+   * @param fileName
+   * @throws ZipException
+   */
+  public void removeFile(String fileName) throws ZipException {
+    if (!isStringNotNullAndNotEmpty(fileName)) {
+      throw new ZipException("file name is empty or null, cannot remove file");
+    }
+
+    removeFiles(Collections.singletonList(fileName));
+  }
+
+  /**
+   * Removes all files from the zip file that match the names in the input list.
+   *
+   * If any of the file is a directory, all the files and directories under this directory
+   * will be removed as well
+   *
+   * If zip file is a split zip file, then this method throws an exception as
+   * zip specification does not allow for updating split zip archives.
+   *
+   * @param fileNames
+   * @throws ZipException
+   */
+  public void removeFiles(List<String> fileNames) throws ZipException {
+    if (fileNames == null) {
+      throw new ZipException("fileNames list is null");
+    }
+
+    if (fileNames.isEmpty()) {
+      return;
+    }
+
     if (zipModel == null) {
       readZipInfo();
     }
@@ -676,8 +716,90 @@ public class ZipFile {
       throw new ZipException("Zip file format does not allow updating split/spanned files");
     }
 
-    new RemoveEntryFromZipFileTask(progressMonitor, runInThread, zipModel).execute(
-            new RemoveEntryFromZipFileTaskParameters(fileHeader, charset));
+    new RemoveFilesFromZipTask(zipModel, headerWriter, buildAsyncParameters()).execute(
+        new RemoveFilesFromZipTaskParameters(fileNames, charset));
+  }
+
+  /**
+   * Renames file name of the entry represented by file header. If the file name in the input file header does not
+   * match any entry in the zip file, the zip file will not be modified.
+   *
+   * If the file header is a folder in the zip file, all sub-files and sub-folders in the zip file will also be renamed.
+   *
+   * Zip file format does not allow modifying a split zip file. Therefore if the zip file being dealt with is a split
+   * zip file, this method throws an exception
+   *
+   * @param fileHeader file header to be changed
+   * @param newFileName the file name that has to be changed to
+   * @throws ZipException if fileHeader is null or newFileName is null or empty or if the zip file is a split file
+   */
+  public void renameFile(FileHeader fileHeader, String newFileName) throws ZipException {
+    if (fileHeader == null) {
+      throw new ZipException("File header is null");
+    }
+
+    renameFile(fileHeader.getFileName(), newFileName);
+  }
+
+  /**
+   * Renames file name of the entry represented by input fileNameToRename. If there is no entry in the zip file matching
+   * the file name as in fileNameToRename, the zip file will not be modified.
+   *
+   * If the entry with fileNameToRename is a folder in the zip file, all sub-files and sub-folders in the zip file will
+   * also be renamed. For a folder, the fileNameToRename has to end with zip file separator "/". For example, if a
+   * folder name "some-folder-name" has to be modified to "new-folder-name", then value of fileNameToRename should be
+   * "some-folder-name/". If newFileName does not end with a separator, zip4j will add a separator.
+   *
+   * Zip file format does not allow modifying a split zip file. Therefore if the zip file being dealt with is a split
+   * zip file, this method throws an exception
+   *
+   * @param fileNameToRename file name in the zip that has to be renamed
+   * @param newFileName the file name that has to be changed to
+   * @throws ZipException if fileNameToRename is empty or newFileName is empty or if the zip file is a split file
+   */
+  public void renameFile(String fileNameToRename, String newFileName) throws ZipException {
+    if (!Zip4jUtil.isStringNotNullAndNotEmpty(fileNameToRename)) {
+      throw new ZipException("file name to be changed is null or empty");
+    }
+
+    if (!Zip4jUtil.isStringNotNullAndNotEmpty(newFileName)) {
+      throw new ZipException("newFileName is null or empty");
+    }
+
+    renameFiles(Collections.singletonMap(fileNameToRename, newFileName));
+  }
+
+  /**
+   * Renames all the entries in the zip file that match the keys in the map to their corresponding values in the map. If
+   * there are no entries matching any of the keys from the map, the zip file is not modified.
+   *
+   * If any of the entry in the map represents a folder, all files and folders will be renamed so that their parent
+   * represents the renamed folder.
+   *
+   * Zip file format does not allow modifying a split zip file. Therefore if the zip file being dealt with is a split
+   * zip file, this method throws an exception
+   *
+   * @param fileNamesMap map of file names that have to be changed with values in the map being the name to be changed to
+   * @throws ZipException if map is null or if the zip file is a split file
+   */
+  public void renameFiles(Map<String, String> fileNamesMap) throws ZipException {
+    if (fileNamesMap == null) {
+      throw new ZipException("fileNamesMap is null");
+    }
+
+    if (fileNamesMap.size() == 0) {
+      return;
+    }
+
+    readZipInfo();
+
+    if (zipModel.isSplitArchive()) {
+      throw new ZipException("Zip file format does not allow updating split/spanned files");
+    }
+
+    AsyncZipTask.AsyncTaskParameters asyncTaskParameters = buildAsyncParameters();
+    new RenameFilesTask(zipModel, headerWriter, new RawIO(), charset, asyncTaskParameters).execute(
+        new RenameFilesTaskParameters(fileNamesMap));
   }
 
   /**
@@ -702,7 +824,7 @@ public class ZipFile {
       throw new ZipException("zip model is null, corrupt zip file?");
     }
 
-    new MergeSplitZipFileTask(progressMonitor, runInThread, zipModel).execute(
+    new MergeSplitZipFileTask(zipModel, buildAsyncParameters()).execute(
             new MergeSplitZipFileTaskParameters(outputZipFile, charset));
   }
 
@@ -731,7 +853,7 @@ public class ZipFile {
       throw new ZipException("end of central directory is null, cannot set comment");
     }
 
-    new SetCommentTask(progressMonitor, runInThread, zipModel).execute(
+    new SetCommentTask(zipModel, buildAsyncParameters()).execute(
             new SetCommentTaskTaskParameters(comment, charset));
   }
 
@@ -846,7 +968,7 @@ public class ZipFile {
       throw new ZipException("no read access for the input zip file");
     }
 
-    try (RandomAccessFile randomAccessFile = new RandomAccessFile(zipFile, RandomAccessFileMode.READ.getValue())) {
+    try (RandomAccessFile randomAccessFile = initializeRandomAccessFileForHeaderReading()) {
       HeaderReader headerReader = new HeaderReader();
       zipModel = headerReader.readAllHeaders(randomAccessFile, charset);
       zipModel.setZipFile(zipFile);
@@ -867,6 +989,29 @@ public class ZipFile {
     zipModel.setZipFile(zipFile);
   }
 
+  private RandomAccessFile initializeRandomAccessFileForHeaderReading() throws IOException {
+    if (isNumberedSplitFile(zipFile)) {
+      File[] allSplitFiles = FileUtils.getAllSortedNumberedSplitFiles(zipFile);
+      NumberedSplitRandomAccessFile numberedSplitRandomAccessFile =  new NumberedSplitRandomAccessFile(zipFile,
+          RandomAccessFileMode.READ.getValue(), allSplitFiles);
+      numberedSplitRandomAccessFile.openLastSplitFileForReading();
+      return numberedSplitRandomAccessFile;
+    }
+
+    return new RandomAccessFile(zipFile, RandomAccessFileMode.READ.getValue());
+  }
+
+  private AsyncZipTask.AsyncTaskParameters buildAsyncParameters() {
+    if (runInThread) {
+      if (threadFactory == null) {
+        threadFactory = Executors.defaultThreadFactory();
+      }
+      executorService = Executors.newSingleThreadExecutor(threadFactory);
+    }
+
+    return new AsyncZipTask.AsyncTaskParameters(executorService, runInThread, progressMonitor);
+  }
+
   public ProgressMonitor getProgressMonitor() {
     return progressMonitor;
   }
@@ -894,6 +1039,14 @@ public class ZipFile {
     this.charset = charset;
   }
 
+  public void setThreadFactory(ThreadFactory threadFactory) {
+    this.threadFactory = threadFactory;
+  }
+
+  public ExecutorService getExecutorService() {
+    return executorService;
+  }
+
   @Override
   public String toString() {
     return zipFile.toString();
