@@ -18,6 +18,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.nio.file.Files;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
@@ -448,6 +449,32 @@ public class ExtractZipFileIT extends AbstractIT {
     assertThat(zipFile.getFileHeaders()).hasSize(19);
   }
 
+  @Test
+  public void testExtractFileHeaderExtractAllFilesIfFileHeaderIsDirectory() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setIncludeRootFolder(false);
+    zipFile.addFolder(TestUtils.getTestFileFromResources(""), zipParameters);
+
+    zipFile.extractFile(zipFile.getFileHeader("öüäöäö/"), outputFolder.getPath());
+    File outputFile = Paths.get(outputFolder.getPath(), "öüäöäö", "asöäööl").toFile();
+    assertThat(outputFile).exists();
+    ZipFileVerifier.verifyFileContent(TestUtils.getTestFileFromResources("öüäöäö/asöäööl"), outputFile);
+  }
+
+  @Test
+  public void testExtractFileHeaderExtractAllFilesIfFileHeaderIsDirectoryAndRenameFile() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setIncludeRootFolder(false);
+    zipFile.addFolder(TestUtils.getTestFileFromResources(""), zipParameters);
+
+    zipFile.extractFile(zipFile.getFileHeader("öüäöäö/"), outputFolder.getPath(), "new_folder_name/");
+    File outputFile = Paths.get(outputFolder.getPath(), "new_folder_name", "asöäööl").toFile();
+    assertThat(outputFile).exists();
+    ZipFileVerifier.verifyFileContent(TestUtils.getTestFileFromResources("öüäöäö/asöäööl"), outputFile);
+  }
+
   private void addFileToZip(ZipFile zipFile, String fileName, EncryptionMethod encryptionMethod, String password) throws ZipException {
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setEncryptFiles(encryptionMethod != null);
